package org.esupportail.cas.server.util.log;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;

import org.apache.log4j.Category;
import org.apache.log4j.xml.DOMConfigurator;
import org.esupportail.cas.server.GenericHandler;

//---------------------------------------------------------

/**
 * This abstract class offers a clear debug log
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public final class Log {
	
	/**
	 * Constructor (no instanciation).
	 */
	private Log() { }
	
	/**
	 * Logging object.
	 */
	private static Category log = null;
	
	/**
	 * A list to store the names of the classes used for logging.
	 */
	private static List logClasses;
	
	/**
	 * The height of the stack not to watch to indent output.
	 */
	private static int baseStackLevel = 0;
	
	/**
	 * The config filename of the logger
	 */
	private static String confilename = null;
	
	/**
	 * Initialize the logger (done only once).
	 */
	public static void init() {
		if (log == null) {
			System.err.println("initializing CAS GenericHandler Logger...");
			String loggerXmlFileName = getConfigLogFileName("/../LoggerConf.xml");
			if (loggerXmlFileName == null) {
				System.err.println(
						"CAS GenericHandler Logger configuration file is missing, "
						+ "setting default configuration..."
				);
				BasicConfigurator.configure();
				log = Logger.getLogger("CASGH");
			} else {
				// decode string to allow paths with spaces (since version 2.0.4)
				// J-Louis.RENAUD at univ-bpclermont.fr
				String str = null;
				try {
					str = URLDecoder.decode(loggerXmlFileName, "UTF-8");
					DOMConfigurator.configureAndWatch(str);
					log = Category.getRoot();
				} catch (UnsupportedEncodingException e) {
					System.err.println(
							"CAS GenericHandler Logger configuration filename could "
							+ "not be decoded, setting default configuration..."
					);
					BasicConfigurator.configure();
					log = Logger.getLogger("CASGH");
				}
			}
			// set the names of the classes that should be skipped in the execution stack
			logClasses = new LinkedList();
			logClasses.add("org.esupportail.cas.server.util.log.Log");
			logClasses.add("org.esupportail.cas.server.util.log.Debug");
			// get the base stack lvel
			baseStackLevel = getCallingStackLevel();
			System.err.println("done.");
		}
	}
	
	/**
	 * Retrieve the absolute filename of the file storing the configuration of Logger.
	 *
	 * @param filename the name of the configuration file (resource-dependent)
	 *
	 * @return a string.
	 */
	private static String getConfigLogFileName(final String filename) {
		java.net.URL resourceURL;
		
		if (confilename == null) {
			try {
				resourceURL = GenericHandler.class.getResource(filename);
				// if configuration file is missing an exception is thrown and the cas server crashes
				confilename = resourceURL.getFile();
			} catch (NullPointerException e) {
				// resourceURL was null
				System.err.println("configuration file LoggerConf.xml is missing!");
				return null;
			}
		}
		return confilename;
	}
	
	/**
	 * Retrieve the level of the calling method in the execution stack.
	 * 
	 * @return an integer.
	 */
	private static int getCallingStackLevel() {
		StackTraceElement[] stack = new Exception().getStackTrace();
		
		String className = null;
		String methodName = null;
		String fileName = null;
		int lineNumber = 0;
		for (int i = 0; i < stack.length; i++) {
			StackTraceElement ste = stack[i];
			if (logClasses.contains(ste.getClassName())) {
				continue;
			}
			return stack.length - i;
		}
		return -1;
	}
	
	/**
	 * Retrieve the stack element corresponding to the calling method.
	 * 
	 * @return a StackTraceElement instance.
	 */
	private static StackTraceElement getCallingStackElement() {
		StackTraceElement[] stack = new Exception().getStackTrace();
		
		String className = null;
		String methodName = null;
		String fileName = null;
		int lineNumber = 0;
		for (int i = 0; i < stack.length; i++) {
			StackTraceElement ste = stack[i];
			if (logClasses.contains(ste.getClassName())) {
				continue;
			}
			return ste;
		}
		return null;
	}
	
	/**
	 * Output a string as an information (INFO).
	 *
	 * @param str a string
	 */
	public static void info(final String str) {
		init();
		log.info(str);
	}
	
	/**
	 * Output a string as a warning (WARN).
	 *
	 * @param str a string
	 */
	public static void warn(final String str) {
		init();
		log.warn(str);
	}
	
	/**
	 * Output a string as an error (SEVERE).
	 *
	 * @param str a string
	 */
	public static void error(final String str) {
		init();
		log.error(str);
	}
	
	/**
	 * Get indentation prefix (depends on the current stack).
	 *
	 * @return a string.
	 */
	private static String getIndentPrefix() {
		return getIndentPrefix(0);
	}
	
	/**
	 * Get indentation prefix (depends on the current stack), modifying the level.
	 * 
	 * @param level 0 to set normal indentation, -1 to remove one level, ...
	 *
	 * @return a string.
	 */
	private static String getIndentPrefix(final int level) {
		StringBuffer buf = new StringBuffer();
		int localLevel = getCallingStackLevel() - baseStackLevel + level;
		for (int i = 0; i < localLevel; i++) {
			buf.append("|    ");
		}
		return buf.toString();
	}
	
	/**
	 * Output a string as a trace (DEBUG).
	 *
	 * @param str a string
	 */
	public static void debug(final String str) {
		debug(str, 0);
	}
	
	/**
	 * Output a string as a trace (DEBUG), modifying the indentation.
	 *
	 * @param str a string
	 * @param level 0 to set normal indentation, -1 to remove one level, ...
	 */
	public static void debug(final String str, final int level) {
		init();
		log.debug(getIndentPrefix(level) + str);
	}
	
	/**
	 * Debugging (beginning of the execution of a method).
	 */
	public static void traceBegin() {
		StackTraceElement ste = getCallingStackElement();
	    String[] classNameElements = ste.getClassName().split("\\.");
	    String classBaseName = classNameElements[classNameElements.length - 1];
		StringBuffer buf = new StringBuffer("=> " + classBaseName + "::" + ste.getMethodName() + "()");
		if (ste.getFileName() != null) {
			buf.append(" [" + ste.getFileName() + ":" + String.valueOf(ste.getLineNumber()) + "]");
		}
		debug(buf.toString(), -1);
	}
	
	/**
	 * Debugging (a string at the end of the execution of a method).
	 *
	 * @param str a string
	 */
	public static void traceEnd(final String str) {
		debug("<= " + str, -1);
	}

}

